package org.nakedobjects.nos.client.web.server;

import java.util.ArrayList;
import java.util.EventListener;
import java.util.Iterator;
import java.util.List;

import org.apache.log4j.Logger;
import org.mortbay.jetty.Server;
import org.mortbay.jetty.handler.ContextHandler;
import org.mortbay.jetty.handler.HandlerList;
import org.mortbay.jetty.handler.ResourceHandler;
import org.mortbay.jetty.servlet.ServletHandler;
import org.mortbay.jetty.servlet.ServletHolder;
import org.mortbay.jetty.servlet.SessionHandler;
import org.nakedobjects.nos.client.web.servlet.Controller;
import org.nakedobjects.nos.client.web.servlet.Logon;


public class NakedObjectsWebServer {
    private final static Logger LOG = Logger.getLogger(NakedObjectsWebServer.class);

    protected static void init(int port) {
        init(port, (String) null);
    }

    protected static void init(int port, String resourceBase) {
        init(port, null, resourceBase);
    }

    private static void init(int port, EventListener servletInitializer, String resourceBase) {
        Server server = new Server(port);

        ContextHandler context = new ContextHandler("/");
        context.setClassLoader(Thread.currentThread().getContextClassLoader());

        if (servletInitializer != null) {
            context.addEventListener(servletInitializer);
        }
        server.addHandler(context);

        HandlerList servletAndResourceHandler = new HandlerList();

        /*
         * The following handlers allow for the different places files are to be found in the different file
         * layouts. These are for Maven/Eclipse, and the demo and examples directories in the distributions.
         * 
         * See also ImageLookup class, which has similar code.
         */
        List resourceBases = new ArrayList();
        resourceBases.add("../nos-web-app/src/main/webapp");
        resourceBases.add("./src/main/webapp");
        resourceBases.add("./src/main/resources");
        resourceBases.add("./web");
        resourceBases.add(".");
        
        
        if (resourceBase != null) {
            resourceBases.add(resourceBase);
        }
        Iterator iter = resourceBases.iterator();
        while (iter.hasNext()) {
            String rb = (String) iter.next();
            ResourceHandler resourceHandler = new ResourceHandler();
            resourceHandler.setResourceBase(rb);
            resourceHandler.setWelcomeFiles(new String[]{"logon.app"});
            servletAndResourceHandler.addHandler(resourceHandler);
        }

        SessionHandler sessionHandler = new SessionHandler();
        servletAndResourceHandler.addHandler(sessionHandler);

        ServletHandler servletHandler = new ServletHandler();
        sessionHandler.setHandler(servletHandler);
        
        context.setHandler(servletAndResourceHandler);

        servletHandler.addServletWithMapping(new ServletHolder(Logon.class), "/logon.app");
        servletHandler.addServletWithMapping(new ServletHolder(Controller.class), "*.app");

        LOG.info("Starting web server: open a web browser and use URL http://localhost:8080/logon.app to connect");
        try {
            server.start();
     //       server.join();
        } catch (RuntimeException e) {
            throw e;
        } catch (Exception e) {
            throw new WebServerException("Web server failed to start", e);
        }
    }
}
