package org.nakedobjects.nos.client.web.server;

import java.io.File;

import org.nakedobjects.noa.client.NakedObjectClient;
import org.nakedobjects.noa.security.SessionManager;
import org.nakedobjects.noa.security.AuthenticationRequest;
import org.nakedobjects.noa.security.ConnectionManager;
import org.nakedobjects.nof.core.conf.Configuration;
import org.nakedobjects.nof.core.context.NakedObjectsContext;
import org.nakedobjects.nos.client.web.component.ImageLookup;
import org.nakedobjects.nos.client.web.servlet.SystemAccess;


class WebBrowserClient implements NakedObjectClient {
    private ConnectionManager connectionManager;
    private SessionManager sessionManager;
    private boolean showExplorationOptions;
    private boolean authenticateUser;

    public void run() {
        String resourceBaseDir = NakedObjectsContext.getConfiguration().getString(Configuration.ROOT + "viewer.web.resourceBase");
        if (resourceBaseDir != null) {
            String imagesDir = resourceBaseDir + File.separator + "images";
            if (fileExists(imagesDir)) {
                ImageLookup.setImageDirectory(imagesDir);
            }
        } else {
            /*
             * The following three directories allow for the different places files are to be found within the
             * different file layouts. These are for Maven/Eclipse, and the demo and examples directories in
             * the distributions.
             */
            if (fileExists("images")) {
                ImageLookup.setImageDirectory("images");
            } else if (fileExists("web/images")) {
                ImageLookup.setImageDirectory("web/images");
            } else {
                ImageLookup.setImageDirectory("src/main/resources/images");
            }
        }
        new SystemAccess(connectionManager, sessionManager, authenticateUser, showExplorationOptions);
        NakedObjectsWebServer.init(8080, resourceBaseDir);

    }

    private boolean fileExists(String fileName) {
        return new File(fileName).exists();
    }

    public void setExploration(boolean showExplorationOptions) {
        this.showExplorationOptions = showExplorationOptions;
    }

    public void setTitle(String title) {}

    public void shutdown() {}

    public void setAuthenticate(boolean authenticateUser) {
        this.authenticateUser = authenticateUser;
    }

    public void setConnectionManager(ConnectionManager connectionManager) {
        this.connectionManager = connectionManager;
    }
    
    public void setSessionManager(final SessionManager sessionManager) {
        this.sessionManager = sessionManager;
    }

    public void setAuthenticationRequestViaArgs(AuthenticationRequest authenticationRequest) {
    // does nothing.
    }

}

// Copyright (c) Naked Objects Group Ltd.
